@@ -56,9 +56,9 @@
 #include "contact-delegate-compact.h"
 #include "contact-overlays.h"
 
-#define PREFERRED_TEXTCHAT_HANDLER "org.freedesktop.Telepathy.Client.KDE.TextUi"
-#define PREFERRED_FILETRANSFER_HANDLER "org.freedesktop.Telepathy.Client.KDE.FileTransfer"
-#define PREFERRED_AUDIO_VIDEO_HANDLER "org.freedesktop.Telepathy.Client.KDE.CallUi"
+#define PREFERRED_TEXTCHAT_HANDLER "org.freedesktop.Telepathy.Client.KTp.TextUi"
+#define PREFERRED_FILETRANSFER_HANDLER "org.freedesktop.Telepathy.Client.KTp.FileTransfer"
+#define PREFERRED_AUDIO_VIDEO_HANDLER "org.freedesktop.Telepathy.Client.KTp.CallUi"
 #define PREFERRED_RFB_HANDLER "org.freedesktop.Telepathy.Client.krfb_rfb_handler"
 
 ContactListWidget::ContactListWidget(QWidget *parent)
@@ -77,8 +77,6 @@ ContactListWidget::ContactListWidget(QWidget *parent)
     d->groupsModel = new GroupsModel(d->model, this);
     d->modelFilter = new AccountsFilterModel(this);
     d->modelFilter->setDynamicSortFilter(true);
-    d->modelFilter->clearFilterString();
-    d->modelFilter->setFilterCaseSensitivity(Qt::CaseInsensitive);
     d->modelFilter->setSortRole(Qt::DisplayRole);
 
     setModel(d->modelFilter);
@@ -102,14 +100,14 @@ ContactListWidget::ContactListWidget(QWidget *parent)
     viewport()->setAcceptDrops(true);
     setDropIndicatorShown(true);
 
-    if (guiConfigGroup.readEntry("selected_delegate", "full") == QLatin1String("compact")) {
+    if (guiConfigGroup.readEntry("selected_delegate", "compact") == QLatin1String("compact")) {
         setItemDelegate(d->compactDelegate);
     } else {
         setItemDelegate(d->delegate);
     }
 
     addOverlayButtons();
-    emit enableOverlays(guiConfigGroup.readEntry("selected_delegate", "full") == QLatin1String("full"));
+    emit enableOverlays(guiConfigGroup.readEntry("selected_delegate", "compact") == QLatin1String("full"));
 
     connect(this, SIGNAL(clicked(QModelIndex)),
             this, SLOT(onContactListClicked(QModelIndex)));
@@ -124,7 +122,7 @@ ContactListWidget::ContactListWidget(QWidget *parent)
 
 ContactListWidget::~ContactListWidget()
 {
-
+    delete d_ptr;
 }
 
 void ContactListWidget::setAccountManager(const Tp::AccountManagerPtr &accountManager)
@@ -176,6 +174,7 @@ void ContactListWidget::showSettingsKCM()
     }
 
     dialog->addModule("kcm_ktp_accounts");
+    dialog->addModule("kcm_ktp_integration_module");
 
     dialog->setAttribute(Qt::WA_DeleteOnClose);
     dialog->exec();
@@ -259,11 +258,11 @@ void ContactListWidget::addOverlayButtons()
 {
     Q_D(ContactListWidget);
 
-    TextChannelContactOverlay*  textOverlay  = new TextChannelContactOverlay(this);
-    AudioChannelContactOverlay* audioOverlay = new AudioChannelContactOverlay(this);
-    VideoChannelContactOverlay* videoOverlay = new VideoChannelContactOverlay(this);
+    TextChannelContactOverlay *textOverlay  = new TextChannelContactOverlay(this);
+    AudioChannelContactOverlay *audioOverlay = new AudioChannelContactOverlay(this);
+    VideoChannelContactOverlay *videoOverlay = new VideoChannelContactOverlay(this);
 
-    FileTransferContactOverlay* fileOverlay  = new FileTransferContactOverlay(this);
+    FileTransferContactOverlay *fileOverlay  = new FileTransferContactOverlay(this);
     DesktopSharingContactOverlay *desktopOverlay = new DesktopSharingContactOverlay(this);
 
     d->delegate->installOverlay(textOverlay);
@@ -272,20 +271,8 @@ void ContactListWidget::addOverlayButtons()
     d->delegate->installOverlay(fileOverlay);
     d->delegate->installOverlay(desktopOverlay);
 
-    textOverlay->setView(this);
-    textOverlay->setActive(true);
-
-    audioOverlay->setView(this);
-    audioOverlay->setActive(true);
-
-    videoOverlay->setView(this);
-    videoOverlay->setActive(true);
-
-    fileOverlay->setView(this);
-    fileOverlay->setActive(true);
-
-    desktopOverlay->setView(this);
-    desktopOverlay->setActive(true);
+    d->delegate->setViewOnAllOverlays(this);
+    d->delegate->setAllOverlaysActive(true);
 
     connect(textOverlay, SIGNAL(overlayActivated(QModelIndex)),
             d->delegate, SLOT(hideStatusMessageSlot(QModelIndex)));
@@ -342,14 +329,14 @@ void ContactListWidget::toggleOfflineContacts(bool show)
     Q_D(ContactListWidget);
 
     d->showOffline = show;
-    d->modelFilter->setShowOfflineUsers(show);
+    d->modelFilter->setPresenceTypeFilterFlags(show ? AccountsFilterModel::DoNotFilterByPresence : AccountsFilterModel::ShowOnlyConnected);
 }
 
 void ContactListWidget::toggleSortByPresence(bool sort)
 {
     Q_D(ContactListWidget);
 
-    d->modelFilter->setSortByPresence(sort);
+    d->modelFilter->setSortMode(sort ? AccountsFilterModel::SortByPresence : AccountsFilterModel::DoNotSort);
 }
 
 void ContactListWidget::startTextChannel(ContactModelItem *contactItem)
@@ -364,7 +351,7 @@ void ContactListWidget::startTextChannel(ContactModelItem *contactItem)
     Tp::AccountPtr account = d->model->accountForContactItem(contactItem);
 
     Tp::ChannelRequestHints hints;
-    hints.setHint("org.kde.telepathy","forceRaiseWindow", QVariant(true));
+    hints.setHint("org.freedesktop.Telepathy.ChannelRequest","DelegateToPreferredHandler", QVariant(true));
 
     Tp::PendingChannelRequest *channelRequest = account->ensureTextChat(contact,
                                                                         QDateTime::currentDateTime(),
@@ -385,9 +372,9 @@ void ContactListWidget::startAudioChannel(ContactModelItem *contactItem)
 
     Tp::AccountPtr account = d->model->accountForContactItem(contactItem);
 
-    Tp::PendingChannelRequest *channelRequest = account->ensureStreamedMediaAudioCall(contact,
-                                                                                      QDateTime::currentDateTime(),
-                                                                                      PREFERRED_AUDIO_VIDEO_HANDLER);
+    Tp::PendingChannelRequest *channelRequest = account->ensureAudioCall(contact,
+            QLatin1String("audio"), QDateTime::currentDateTime(), PREFERRED_AUDIO_VIDEO_HANDLER);
+
     connect(channelRequest, SIGNAL(finished(Tp::PendingOperation*)),
             SIGNAL(genericOperationFinished(Tp::PendingOperation*)));
 }
@@ -403,9 +390,10 @@ void ContactListWidget::startVideoChannel(ContactModelItem *contactItem)
 
     Tp::AccountPtr account = d->model->accountForContactItem(contactItem);
 
-    Tp::PendingChannelRequest* channelRequest = account->ensureStreamedMediaVideoCall(contact, true,
-                                                                                      QDateTime::currentDateTime(),
-                                                                                      PREFERRED_AUDIO_VIDEO_HANDLER);
+    Tp::PendingChannelRequest* channelRequest = account->ensureAudioVideoCall(contact,
+            QLatin1String("audio"), QLatin1String("video"),
+            QDateTime::currentDateTime(), PREFERRED_AUDIO_VIDEO_HANDLER);
+
     connect(channelRequest, SIGNAL(finished(Tp::PendingOperation*)),
             SIGNAL(genericOperationFinished(Tp::PendingOperation*)));
 }
@@ -531,12 +519,48 @@ void ContactListWidget::onSwitchToCompactView()
     guiConfigGroup.config()->sync();
 }
 
+void ContactListWidget::onShowAllContacts()
+{
+    Q_D(ContactListWidget);
+
+    d->modelFilter->setSubscriptionStateFilterFlags(AccountsFilterModel::DoNotFilterBySubscription);
+
+    KSharedConfigPtr config = KGlobal::config();
+    KConfigGroup guiConfigGroup(config, "GUI");
+    guiConfigGroup.writeEntry("shown_contacts", "all");
+    guiConfigGroup.config()->sync();
+}
+
+void ContactListWidget::onShowUnblockedContacts()
+{
+    Q_D(ContactListWidget);
+
+    d->modelFilter->setSubscriptionStateFilterFlags(AccountsFilterModel::HideBlocked);
+
+    KSharedConfigPtr config = KGlobal::config();
+    KConfigGroup guiConfigGroup(config, "GUI");
+    guiConfigGroup.writeEntry("shown_contacts", "unblocked");
+    guiConfigGroup.config()->sync();
+}
+
+void ContactListWidget::onShowBlockedContacts()
+{
+    Q_D(ContactListWidget);
+
+    d->modelFilter->setSubscriptionStateFilterFlags(AccountsFilterModel::ShowOnlyBlocked);
+
+    KSharedConfigPtr config = KGlobal::config();
+    KConfigGroup guiConfigGroup(config, "GUI");
+    guiConfigGroup.writeEntry("shown_contacts", "blocked");
+    guiConfigGroup.config()->sync();
+}
+
 void ContactListWidget::setFilterString(const QString& string)
 {
     Q_D(ContactListWidget);
 
-    d->modelFilter->setShowOfflineUsers(!string.isEmpty() || d->showOffline);
-    d->modelFilter->setFilterString(string);
+    d->modelFilter->setPresenceTypeFilterFlags(string.isEmpty() && !d->showOffline ? AccountsFilterModel::ShowOnlyConnected : AccountsFilterModel::DoNotFilterByPresence);
+    d->modelFilter->setDisplayNameFilterString(string);
 }
 
 void ContactListWidget::setDropIndicatorRect(const QRect &rect)
@@ -549,6 +573,17 @@ void ContactListWidget::setDropIndicatorRect(const QRect &rect)
     }
 }
 
+bool ContactListWidget::event(QEvent *event)
+{
+    Q_D(ContactListWidget);
+    if (event->type() == QEvent::Leave && d->delegate) {
+        d->delegate->reshowStatusMessageSlot();
+        return true;
+    }
+
+    return QTreeView::event(event);
+}
+
 void ContactListWidget::mousePressEvent(QMouseEvent *event)
 {
     Q_D(ContactListWidget);
@@ -673,10 +708,12 @@ void ContactListWidget::dropEvent(QDropEvent *event)
 
             kDebug() << contact->contact().data()->alias() << "added to group" << group;
 
-            Tp::PendingOperation *op = contact->contact().data()->addToGroup(group);
+            if (!group.isEmpty()) {
+                Tp::PendingOperation *op = contact->contact().data()->addToGroup(group);
 
-            connect(op, SIGNAL(finished(Tp::PendingOperation*)),
-                    this, SIGNAL(genericOperationFinished(Tp::PendingOperation*)));
+                connect(op, SIGNAL(finished(Tp::PendingOperation*)),
+                        this, SIGNAL(genericOperationFinished(Tp::PendingOperation*)));
+            }
         }
         event->acceptProposedAction();
     } else {
