@@ -39,6 +39,7 @@
 #include <KTp/Models/accounts-model.h>
 #include <KTp/Models/contact-model-item.h>
 #include <KTp/Models/groups-model-item.h>
+#include <KTp/Widgets/add-contact-dialog.h>
 
 #include <KDebug>
 #include <KDialog>
@@ -54,7 +55,6 @@
 #include "ui_main-widget.h"
 #include "account-buttons-panel.h"
 #include "contact-list-application.h"
-#include "dialogs/add-contact-dialog.h"
 #include "dialogs/join-chat-room-dialog.h"
 #include "tooltips/tooltipmanager.h"
 #include "context-menu.h"
@@ -82,11 +82,11 @@ MainWidget::MainWidget(QWidget *parent)
 
     m_toolBar->setToolButtonStyle(Qt::ToolButtonIconOnly);
 
-    m_addContactAction = new KAction(KIcon("list-add-user"), i18n("Add new contacts..."), this);
+    m_addContactAction = new KAction(KIcon("list-add-user"), i18n("Add New Contacts..."), this);
 
     m_toolBar->addAction(m_addContactAction);
 
-    m_groupContactsAction = new KAction(KIcon("user-group-properties"), i18n("Show/Hide groups"), this);
+    m_groupContactsAction = new KAction(KIcon("user-group-properties"), i18n("Show/Hide Groups"), this);
     m_groupContactsAction->setCheckable(true);
     m_groupContactsAction->setChecked(true);
     //TODO: Toggle the tooltip with the button? eg. once its Show, after click its Hide .. ?
@@ -124,21 +124,21 @@ MainWidget::MainWidget(QWidget *parent)
 
 
 
-    m_showOfflineAction = new KAction(KIcon("meeting-attending-tentative"), i18n("Hide/Show offline users"), this);
+    m_showOfflineAction = new KAction(KIcon("meeting-attending-tentative"), i18n("Hide/Show Offline Users"), this);
     m_showOfflineAction->setCheckable(true);
     m_showOfflineAction->setChecked(false);
 
     m_toolBar->addAction(m_showOfflineAction);
 
     m_sortByPresenceAction = new KDualAction(i18n("List is sorted by name. Click to sort by presence."),
-                                             i18n("List is sorted by presence. Click to sort by name."), 
+                                             i18n("List is sorted by presence. Click to sort by name."),
                                              this);
     m_sortByPresenceAction->setActiveIcon(KIcon("sort-presence"));
     m_sortByPresenceAction->setInactiveIcon(KIcon("sort-name"));
 
     m_toolBar->addAction(m_sortByPresenceAction);
 
-    m_searchContactAction = new KAction(KIcon("edit-find-user"), i18n("Find contact"), this );
+    m_searchContactAction = new KAction(KIcon("edit-find-user"), i18n("Find Contact"), this );
     m_searchContactAction->setShortcut(KStandardShortcut::find());
     m_searchContactAction->setCheckable(true);
     m_searchContactAction->setChecked(false);
@@ -155,37 +155,71 @@ MainWidget::MainWidget(QWidget *parent)
     settingsButton->setPopupMode(QToolButton::InstantPopup);
 
     KMenu *settingsButtonMenu = new KMenu(settingsButton);
-    settingsButtonMenu->addAction(i18n("Configure accounts..."), m_contactsListView, SLOT(showSettingsKCM()));
+    settingsButtonMenu->addAction(i18n("Instant Messaging Settings..."), m_contactsListView, SLOT(showSettingsKCM()));
 
     QActionGroup *delegateTypeGroup = new QActionGroup(this);
     delegateTypeGroup->setExclusive(true);
 
     KMenu *setDelegateTypeMenu = new KMenu(settingsButtonMenu);
-    setDelegateTypeMenu->setTitle(i18n("Contact list type"));
-    delegateTypeGroup->addAction(setDelegateTypeMenu->addAction(i18n("Use full list"),
+    setDelegateTypeMenu->setTitle(i18n("Contact List Type"));
+    delegateTypeGroup->addAction(setDelegateTypeMenu->addAction(i18n("Use Full List"),
                                                                 m_contactsListView, SLOT(onSwitchToFullView())));
     delegateTypeGroup->actions().last()->setCheckable(true);
 
-    if (guiConfigGroup.readEntry("selected_delegate", "full") == QLatin1String("full")) {
+    if (guiConfigGroup.readEntry("selected_delegate", "compact") == QLatin1String("full")) {
         delegateTypeGroup->actions().last()->setChecked(true);
     }
 
-    delegateTypeGroup->addAction(setDelegateTypeMenu->addAction(i18n("Use compact list"),
+    delegateTypeGroup->addAction(setDelegateTypeMenu->addAction(i18n("Use Compact List"),
                                                                 m_contactsListView, SLOT(onSwitchToCompactView())));
     delegateTypeGroup->actions().last()->setCheckable(true);
 
-    if (guiConfigGroup.readEntry("selected_delegate", "full") == QLatin1String("compact")) {
+    if (guiConfigGroup.readEntry("selected_delegate", "compact") == QLatin1String("compact")) {
         delegateTypeGroup->actions().last()->setChecked(true);
     }
 
     settingsButtonMenu->addMenu(setDelegateTypeMenu);
 
+    KMenu *setBlockedFilterMenu = new KMenu(settingsButtonMenu);
+    setBlockedFilterMenu->setTitle(i18n("Shown Contacts"));
+
+    QActionGroup *blockedFilterGroup = new QActionGroup(this);
+    blockedFilterGroup->setExclusive(true);
+
+    QString shownContacts = guiConfigGroup.readEntry("shown_contacts", "unblocked");
+
+    blockedFilterGroup->addAction(setBlockedFilterMenu->addAction(i18n("Show all contacts"),
+                                                                  m_contactsListView, SLOT(onShowAllContacts())));
+    blockedFilterGroup->actions().last()->setCheckable(true);
+    if (shownContacts == QLatin1String("all")) {
+        blockedFilterGroup->actions().last()->setChecked(true);
+    }
+
+    blockedFilterGroup->addAction(setBlockedFilterMenu->addAction(i18n("Show unblocked contacts"),
+                                                                  m_contactsListView, SLOT(onShowUnblockedContacts())));
+    blockedFilterGroup->actions().last()->setCheckable(true);
+    if (shownContacts == QLatin1String("unblocked")) {
+        blockedFilterGroup->actions().last()->setChecked(true);
+    }
+
+    blockedFilterGroup->addAction(setBlockedFilterMenu->addAction(i18n("Show blocked contacts"),
+                                                                  m_contactsListView, SLOT(onShowBlockedContacts())));
+    blockedFilterGroup->actions().last()->setCheckable(true);
+    if (shownContacts == QLatin1String("blocked")) {
+        blockedFilterGroup->actions().last()->setChecked(true);
+    }
+
+    settingsButtonMenu->addMenu(setBlockedFilterMenu);
+
     if (guiConfigGroup.readEntry("selected_presence_chooser", "global") == QLatin1String("global")) {
 //         //hide account buttons and show global presence
          onUseGlobalPresenceTriggered();
     }
 
-    settingsButtonMenu->addAction(i18n("Join chat room"), this, SLOT(onJoinChatRoomRequested()));
+    // Restore window geometry
+    restoreGeometry(guiConfigGroup.readEntry("window_geometry", QByteArray()));
+
+    settingsButtonMenu->addAction(i18n("Join Chat Room..."), this, SLOT(onJoinChatRoomRequested()));
     settingsButtonMenu->addSeparator();
     settingsButtonMenu->addMenu(helpMenu());
 
@@ -296,7 +330,7 @@ void MainWidget::showMessageToUser(const QString& text, const MainWidget::System
 }
 
 void MainWidget::onAddContactRequest() {
-    QWeakPointer<AddContactDialog> dialog = new AddContactDialog(m_contactsListView->accountsModel(), this);
+    QWeakPointer<KTp::AddContactDialog> dialog = new KTp::AddContactDialog(m_contactsListView->accountsModel(), this);
     if (dialog.data()->exec() == QDialog::Accepted) {
         Tp::AccountPtr account = dialog.data()->account();
         if (account.isNull()) {
@@ -393,6 +427,7 @@ void MainWidget::closeEvent(QCloseEvent* e)
     KSharedConfigPtr config = KGlobal::config();
     KConfigGroup generalConfigGroup(config, "General");
     KConfigGroup notifyConigGroup(config, "Notification Messages");
+    KConfigGroup guiConfigGroup(config, "GUI");
 
     ContactListApplication *app = qobject_cast<ContactListApplication*>(kapp);
     if (!app->isShuttingDown()) {
@@ -429,6 +464,10 @@ void MainWidget::closeEvent(QCloseEvent* e)
         generalConfigGroup.config()->sync();
     }
 
+    // Save window geometry
+    guiConfigGroup.writeEntry("window_geometry", saveGeometry());
+    guiConfigGroup.config()->sync();
+
     KMainWindow::closeEvent(e);
 }
 
